"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BacktraceService = void 0;
var argparse_1 = require("argparse");
var backtrace_node_1 = require("backtrace-node");
var body_parser_1 = require("body-parser");
var compression_1 = __importDefault(require("compression"));
var cors_1 = __importDefault(require("cors"));
var express_1 = __importDefault(require("express"));
var fs_1 = __importDefault(require("fs"));
var helmet_1 = __importDefault(require("helmet"));
var nocache_1 = __importDefault(require("nocache"));
var __1 = require("..");
var logger_1 = require("../log/logger");
var metricsStorage_1 = require("../metrics/metricsStorage");
var errorScope_1 = require("../model/errorScope");
var packageUtils_1 = require("../utils/packageUtils");
var projectValidation_1 = require("./projectValidation");
var serviceConfiguration_1 = require("./serviceConfiguration");
// tslint:disable-next-line: no-var-requires
var packageJson = require('../../package.json');
var BacktraceService = /** @class */ (function () {
    function BacktraceService(name, port, _opts) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        this.name = name;
        this.port = port;
        this._opts = _opts;
        this.coronerdAddress = new Set();
        this.app = (_opts === null || _opts === void 0 ? void 0 : _opts.app) ? _opts.app : (0, express_1.default)();
        this.initializeDefaultArgParser((_a = this._opts) === null || _a === void 0 ? void 0 : _a.serviceMutation);
        this.configuration = BacktraceService.readConfiguration(this.name, this.port, ((_b = this.arguments) === null || _b === void 0 ? void 0 : _b.configuration) || ((_c = this._opts) === null || _c === void 0 ? void 0 : _c.confPath), false, (_d = this._opts) === null || _d === void 0 ? void 0 : _d.packageName);
        // the descriptor field is always available
        // since it can't be igored by the readconfiguration parameters
        // if the descriptor is not available, then the service refuses
        // to start
        this._descriptor = this.configuration.descriptor;
        this.apiPrefix = this._descriptor.resource;
        this._logger = this.initializeLogger();
        /**
         * In previous version of backtrace-service we assumed that MetricsStorage
         * will always exists. To make sure metricsStorage object is always available
         * for services that already use this object from BacktraceService
         * we will pass empty uuid/secret if remoteLog is disabled.
         */
        var _l = this.getMetricsSecret(), uuid = _l.uuid, secret = _l.secret;
        this.metrics = new metricsStorage_1.MetricsStorage(uuid, secret, {
            interval: (_f = (_e = this.configuration) === null || _e === void 0 ? void 0 : _e.metrics) === null || _f === void 0 ? void 0 : _f.interval,
            hostUrl: (_h = (_g = this.configuration) === null || _g === void 0 ? void 0 : _g.metrics) === null || _h === void 0 ? void 0 : _h.hostUrl,
            disableExpressMetrics: (_k = (_j = this.configuration) === null || _j === void 0 ? void 0 : _j.metrics) === null || _k === void 0 ? void 0 : _k.disableExpressMetrics,
        }, this.app, this._logger);
        this.identityManager = new __1.IdentityManager(this._descriptor, this._logger);
        BacktraceService._service = this;
    }
    Object.defineProperty(BacktraceService.prototype, "arguments", {
        /**
         * Returns ArgumentParser instance arguments.
         */
        get: function () {
            if (!this._arguments) {
                this._arguments = this._argumentParser.parse_known_args()[0];
            }
            return this._arguments;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BacktraceService, "serviceVersion", {
        /**
         * Returns service version
         * @returns service version from the process.env variable
         */
        get: function () {
            if (this._serviceVersion) {
                return this._serviceVersion;
            }
            // the requirement for each service is to run via npm run command.
            // If the service is using npm command to startup then the package
            // version is available in the process.env variables. In the pkg
            // package we don't have npm information available. Because of that
            // if the npm package version is not available in process then
            // backtrace-service will try to check package.json in the
            // process.cwd() dir as a default location of the package.json.
            // This is how our services should be configured by default.
            if (process.env.npm_package_version) {
                this._serviceVersion = process.env.npm_package_version;
                return this._serviceVersion;
            }
            var servicePackageJsonPath = packageUtils_1.PackageUtils.packageJsonPath;
            if (!servicePackageJsonPath || !fs_1.default.existsSync(servicePackageJsonPath)) {
                return undefined;
            }
            var version = require(servicePackageJsonPath).version;
            this._serviceVersion = version;
            return this._serviceVersion;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BacktraceService, "backtraceServiceVersion", {
        /**
         * Returns Backtrace-service version
         */
        get: function () {
            return packageJson.version;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Create and startup express application with basic options.
     * By using this method, application will apply all
     * Backtrace service rules
     * @param name service name (required by service layer)
     * @param port service port (required by service layer). Note this
     * service requested port. Service layer might assign a different
     * service port based on the service descriptor.
     * @param opts optional application options
     * @param expressApp - deprecated. Optional express app instance.
     * Please use expressInstance from app instead.
     * By using this parameter you can create express app in different
     * framework and still be able to use Backtrace integrations
     */
    BacktraceService.bootstrap = function (name, port, opts, app) {
        if (!port || !name) {
            throw new Error("Invalid arguments. Name: ".concat(name, ", port: ").concat(port));
        }
        // backward compatibility
        if (app && opts) {
            opts.app = app;
        }
        var service = new BacktraceService(name, port, opts);
        // assign express intsance prepared by backtrace service/passed by user
        var expressApp = service.app;
        this.setupErrorScopeMiddleware(expressApp, opts === null || opts === void 0 ? void 0 : opts.errorScopeOptions);
        this.setupAuth(expressApp);
        this.setupMiddlewares(expressApp, service.apiPrefix, service.identityManager);
        this.setupHealthCheck(expressApp, service.apiPrefix, service.identityManager, {
            healthcheck: opts === null || opts === void 0 ? void 0 : opts.healthcheck,
        });
        this.setupServiceLayer(expressApp, name, port, function (ip) {
            service.coronerdAddress.add(ip);
            if (opts && opts.coronerdCallback) {
                opts.coronerdCallback(ip);
            }
        });
        process.title = name;
        expressApp.on('close', function () {
            service._logger.warn('server is closing');
        });
        return service;
    };
    /**
     * Adds the healthcheck endpoints to the service. Healthchecks allow
     * to verify the service status and confirm the backtrace-node integration
     * setup.
     * @param expressApp current express app
     * @param apiPrefix api prefix
     * @param identityManager identity manager
     * @param options options
     */
    BacktraceService.setupHealthCheck = function (expressApp, apiPrefix, identityManager, opts) {
        var _this = this;
        expressApp.get("".concat(apiPrefix, "/healthcheck"), identityManager.authenticateRequest(), (0, __1.asyncWrapper)(function (_, res) { return __awaiter(_this, void 0, void 0, function () {
            var _a, _b, _c;
            var _d;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        identityManager.bypassProjectAccessCheck(res);
                        _b = (_a = res.status(200)).send;
                        _d = {
                            serviceVersion: this.serviceVersion,
                            backtraceServiceVersion: this.backtraceServiceVersion
                        };
                        if (!(opts === null || opts === void 0 ? void 0 : opts.healthcheck)) return [3 /*break*/, 2];
                        return [4 /*yield*/, opts.healthcheck()];
                    case 1:
                        _c = _e.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        _c = undefined;
                        _e.label = 3;
                    case 3: return [2 /*return*/, _b.apply(_a, [(_d.metadata = _c,
                                _d)])];
                }
            });
        }); }));
        expressApp.post("".concat(apiPrefix, "/healthcheck/report"), identityManager.authenticateRequest(), (0, __1.asyncWrapper)(function (_, res) { return __awaiter(_this, void 0, void 0, function () {
            var client;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        identityManager.bypassProjectAccessCheck(res);
                        client = (0, backtrace_node_1.getBacktraceClient)();
                        if (!client) {
                            return [2 /*return*/, res.status(400).send({
                                    status: 'Error reporting tool is not available',
                                })];
                        }
                        return [4 /*yield*/, client.reportAsync('Service-test')];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, res.status(200).send({
                                status: 'ok',
                            })];
                }
            });
        }); }));
    };
    /**
     * Read and parse service configuration
     * @param name service name
     * @param port service port number
     * @param configurationPath (optional) service configuration path
     * @param ignoreDescriptor (optional) determine if service
     * should create/read service layer service descriptor
     */
    BacktraceService.readConfiguration = function (name, port, configurationPath, ignoreDescriptor, packageName) {
        var serviceConfiguration = new serviceConfiguration_1.ServiceConfiguration();
        var possiblePaths = configurationPath ? [configurationPath] : [];
        possiblePaths.push.apply(possiblePaths, serviceConfiguration.getProjectSettingsPaths(name, packageName));
        var configuration = serviceConfiguration.loadConfiguration(possiblePaths);
        if (!ignoreDescriptor) {
            configuration.descriptor = (0, __1.getDescriptor)(name, port);
        }
        return configuration;
    };
    /**
     * Setup popular Backtrace middlewares
     * @param app express app
     */
    BacktraceService.setupMiddlewares = function (app, prefix, identityManager) {
        var _this = this;
        if (!prefix) {
            prefix = '/';
        }
        app.use((0, body_parser_1.json)({
            verify: function (res, _, buf) {
                res.rawBody = buf;
            },
        }));
        app.use((0, body_parser_1.urlencoded)({
            extended: true,
            verify: function (res, _, buf) {
                res.rawBody = buf;
            },
        }));
        app.use((0, compression_1.default)());
        // Add default Backtrace headers to every service response
        app.use(function (req, res, next) {
            // add service version
            res.set('service-version', _this.serviceVersion);
            // add backtrace-service version
            res.set('bt-service-version', _this.backtraceServiceVersion);
            next();
        });
        // Route hit by pingdom
        app.get(prefix, function (_req, res) {
            identityManager === null || identityManager === void 0 ? void 0 : identityManager.bypassProjectAccessCheck(res);
            res.type('xml');
            res.status(200).send("<html></html>");
        });
    };
    BacktraceService.setupErrorScopeMiddleware = function (app, opts) {
        app.use(function (req, res, next) {
            var errorScope = new errorScope_1.ErrorScope();
            if (opts === null || opts === void 0 ? void 0 : opts.addExpressAttributes) {
                errorScope.addAttributes(function () { return ({
                    originalUrl: req.originalUrl,
                    protocol: req.protocol,
                    query: req.query,
                    headers: req.headers,
                    body: { value: req.body }, // Avoid putting string bodies in attributes
                }); });
            }
            req.errorScope = errorScope;
            next();
        });
    };
    /**
     * Setup express error middleware
     * @param app express app
     */
    BacktraceService.setupErrorMiddleware = function (app, identityManager, overrideLogger) {
        app.use(function (err, req, res, next) {
            // handle a situation when we already sent a response from onError method
            // we shouldn't do that and we should let error middleware handle that
            // but here we wan't to make sure our code won't break up
            // and we won't generate another error
            if (res.headersSent) {
                return;
            }
            var logger = overrideLogger !== null && overrideLogger !== void 0 ? overrideLogger : logger_1.Logger.getLogger();
            // if we reach this part of code, the error middlewa needs to assume
            // the teams validation weren't handled on the route side - for example:
            // when exception were throwed by the 3rd party library when someone sends
            // invalid JSON object. In this situation we should return an error and mark
            // request as validated
            identityManager === null || identityManager === void 0 ? void 0 : identityManager.bypassProjectAccessCheck(res);
            // Backtrace service ApiError contains type statusCode that allows us
            // to determine what type of response we should return
            // in addition to that the error middleware can read a status code from response.status
            // property. This is a popular pattern for axios communication - check other services
            // to learn more how errors throwed by axios are beings handled by them.
            var errorScope = req.errorScope;
            if (err.statusCode || (err.response && err.response.status)) {
                var status_1 = err.statusCode || err.response.status;
                var message = err.message || err.response.message;
                if (status_1 === 500) {
                    var attributes = errorScope === null || errorScope === void 0 ? void 0 : errorScope.getAttributes();
                    if (attributes) {
                        logger.error(err, attributes);
                    }
                    else {
                        logger.error(err);
                    }
                }
                else {
                    logger.warn(message);
                }
                res.status(status_1).send(message);
            }
            else {
                var attributes = errorScope === null || errorScope === void 0 ? void 0 : errorScope.getAttributes();
                if (attributes) {
                    logger.error(err, attributes);
                }
                else {
                    logger.error(err);
                }
                res.status(500).send({
                    error: 'Internal server error',
                });
            }
        });
        app.use(function (req, res, next) {
            // prevent overriding headers.
            if (res.headersSent) {
                return;
            }
            identityManager === null || identityManager === void 0 ? void 0 : identityManager.bypassProjectAccessCheck(res);
            // response json might be required by front-end
            res.status(404).send({
                error: 'Not found',
            });
        });
    };
    /**
     * Setup security middlewares
     * @param app express app
     */
    BacktraceService.setupAuth = function (app) {
        var corsSettings = (0, cors_1.default)({
            exposedHeaders: ['service-version', 'bt-service-version'],
        });
        app.use(corsSettings);
        app.options('*', corsSettings);
        app.use((0, nocache_1.default)());
        app.use((0, helmet_1.default)({
            hsts: false,
        }));
        app.use(projectValidation_1.projectValidation);
    };
    /**
     * Setup service layer integration
     * @param app express app
     */
    BacktraceService.setupServiceLayer = function (app, name, port, coronerdCallback) {
        var serviceDescriptor = this._service
            ? this._service._descriptor
            : undefined;
        // this case handle a situation when we didn't call backtraceService constr
        // here we want to read service descriptor from hard drive - if descr exists
        // we're using here a service name and port instead of service descriptor
        // to avoid extra steps required by service to setup service layer.
        if (!serviceDescriptor && name && port) {
            serviceDescriptor = (0, __1.getDescriptor)(name, port);
        }
        if (!serviceDescriptor) {
            throw new Error('Descriptor is undefined');
        }
        var identityManager = this._service
            ? this._service.identityManager
            : new __1.IdentityManager(serviceDescriptor);
        app.post("".concat(serviceDescriptor.resource, "/service"), identityManager.serviceRequest({
            name: serviceDescriptor.name,
            secret: serviceDescriptor.secret,
            coronerdCallback: coronerdCallback,
        }));
    };
    /**
     * Returns the information if the service is available in the on-premise environment
     * @returns true if the environment is set to on-premise
     */
    BacktraceService.prototype.isOnPremise = function () {
        return this._descriptor.isOnPremise;
    };
    BacktraceService.prototype.start = function (clusterSupport) {
        var _this = this;
        var _a;
        if (clusterSupport === void 0) { clusterSupport = false; }
        if (!this.app) {
            this._logger.error("Express application is not defined. Cannot start application");
            return;
        }
        if (!this._descriptor) {
            this._logger.error("Descriptor object is not defined.\n        Please execute setupServiceLayer method");
            return;
        }
        if (this.port !== this._descriptor.port) {
            this._logger.warn("Requested application port is different than service descriptor port." +
                "Continuing using service descriptor port: ".concat(this._descriptor.port));
        }
        BacktraceService.setupErrorMiddleware(this.app, this.identityManager, (_a = this._opts) === null || _a === void 0 ? void 0 : _a.logger);
        (0, __1.listenDescriptor)(this._descriptor, this.app, this.configuration.ssl, function () {
            _this._logger.info("Application is Running. ".concat(_this._descriptor
                ? "Port: ".concat(_this._descriptor.port, ". Api prefix: ").concat(_this._descriptor.resource)
                : ''));
        }, clusterSupport);
    };
    /**
     * Initialize the service logger.
     * @returns Service logger instance.
     */
    BacktraceService.prototype.initializeLogger = function () {
        var logLevel = this.configuration.logger && this.configuration.logger.logLevel
            ? this.configuration.logger.logLevel
            : undefined;
        var optionsLogger = this._opts && this._opts.logger;
        var logger;
        if (optionsLogger) {
            logger_1.Logger.setLogger(this._opts.logger);
            logger = this._opts.logger;
        }
        if (!logger) {
            logger = new logger_1.Logger(logLevel);
        }
        if (this.isBacktraceIntegrationEnabled()) {
            logger_1.Logger.initializeBacktrace(this.configuration.backtrace);
        }
        return logger;
    };
    /**
     * Initialize default arg parser object.
     */
    BacktraceService.prototype.initializeDefaultArgParser = function (serviceMutation) {
        this._argumentParser = new argparse_1.ArgumentParser();
        this._argumentParser.add_argument('-c', '--configuration', {
            help: 'Override service configuration path',
        });
        if (serviceMutation) {
            serviceMutation(this);
        }
    };
    /**
     * Determine if backtrace-service should use backtrace-node integration
     */
    BacktraceService.prototype.isBacktraceIntegrationEnabled = function () {
        var _a;
        return ((_a = this.configuration.backtrace) === null || _a === void 0 ? void 0 : _a.enabled) === true;
    };
    /**
     * Generate correct credentials for metricsStorage obejct based on configuration file.
     */
    BacktraceService.prototype.getMetricsSecret = function () {
        var _a, _b, _c;
        var isDiagnosticsEnabled = ((_a = this.configuration.metrics) === null || _a === void 0 ? void 0 : _a.enabled) === true;
        if (isDiagnosticsEnabled === false) {
            return {
                uuid: '',
                secret: '',
            };
        }
        return {
            uuid: (_b = this.configuration.metrics) === null || _b === void 0 ? void 0 : _b.uuid,
            secret: (_c = this.configuration.metrics) === null || _c === void 0 ? void 0 : _c.secret,
        };
    };
    return BacktraceService;
}());
exports.BacktraceService = BacktraceService;
//# sourceMappingURL=backtraceService.js.map