import { Logger } from '..';
import { IConfiguration } from './../model/configuration';
/**
 * Backtrace service configuration. Class responsible for managing configuration paths/parsing them.
 */
export declare class ServiceConfiguration<T> {
    private readonly _logger;
    private readonly _isExecutable;
    private readonly _productionMode;
    private _runningInProdEnvironment;
    constructor(_logger?: Logger, _isExecutable?: boolean);
    /**
     * @deprecated Please use instead getProjectSettingsPath. This method is deprecated because
     * of the way how we want to read configuration files. This method will return only one
     * path to service configuration, that might be wrong for complex environment when user want to
     * override default configuration.
     *
     * Select default path for service configuraiton path in different environment
     * By default Backtrace expect to run service from root service directory
     * by using npm commands. In environment different than production
     * @ServiceConfiguration class will read configuration file from root dir.
     * Otherwise from default Backtrace configuration directory
     */
    getProjectSettingsPath(serviceName: string): string;
    /**
     * Select default paths for service configuraiton path in different environment
     * By default Backtrace expect to run service from root service directory
     * by using npm commands. In environment different than production
     * @ServiceConfiguration class will read configuration file from root dir.
     * Otherwise from default Backtrace configuration directory
     */
    getProjectSettingsPaths(serviceName: string, packageName?: string): string[];
    /**
     * Loads service configuration file
     */
    loadConfiguration(configurationPaths: string[]): IConfiguration<T>;
    /**
     * Returns a package name. If the package is running via npm command
     * and the process.env has a package name in the npm variables then
     * the package.json package name will be returned. Otherwise a service
     * name with the "backtrace-" prefix or name of the package available in
     * the package snapshot.
     */
    private getPackageName;
}
