"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PackageUtils = void 0;
var path_1 = __importDefault(require("path"));
var PackageUtils = /** @class */ (function () {
    function PackageUtils() {
    }
    Object.defineProperty(PackageUtils, "isPackage", {
        /**
         * Returns true if a service is running in the package (pkg)
         */
        get: function () {
            return !!process.pkg;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PackageUtils, "packageDir", {
        /**
         * Returns a path to the backtrace package
         */
        get: function () {
            if (!this.isPackage) {
                // the start directory of each service
                return process.cwd();
            }
            var packageName = this.packageName;
            if (!packageName) {
                return undefined;
            }
            return path_1.default.join('/snapshot', packageName);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PackageUtils, "packageName", {
        /**
         * Return a package name in the snapshot
         */
        get: function () {
            if (!this.isPackage) {
                return undefined;
            }
            // ["", "snapshot", "<package_name>"]
            var dirs = __dirname.split('/', 3);
            return dirs[dirs.length - 1];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PackageUtils, "packageJsonPath", {
        /**
         * Returns path to the service package.json file.
         */
        get: function () {
            var packageDir = this.packageDir;
            if (!packageDir) {
                return undefined;
            }
            return path_1.default.join(packageDir, 'package.json');
        },
        enumerable: false,
        configurable: true
    });
    return PackageUtils;
}());
exports.PackageUtils = PackageUtils;
//# sourceMappingURL=packageUtils.js.map